__version__ = "0.8.1"
__description__ = "aws-gate - AWS SSM Session Manager client CLI"
__author__ = "Adam Stevko"
__author_email__ = "adam.stevko@gmail.com"
__url__ = "https://github.com/xen0l/aws-gate"
